import semver from 'semver';
import { getGithubRepo } from './get-github-repo.js';
import { defaultValidLabels } from './valid-labels.js';
function stripTrailingEmptyLine(text) {
    if (text.endsWith('\n\n')) {
        return text.slice(0, -1);
    }
    return text;
}
function isRecord(value) {
    return typeof value === 'object' && value !== null;
}
function getValidLabels(packageInfo) {
    const prLogConfig = packageInfo['pr-log'];
    if (isRecord(prLogConfig) && Array.isArray(prLogConfig.validLabels)) {
        return new Map(prLogConfig.validLabels);
    }
    return defaultValidLabels;
}
function validateVersionNumber(versionNumber) {
    if (versionNumber.length === 0) {
        throw new TypeError('version-number not specified');
    }
    if (semver.valid(versionNumber) === null) {
        throw new Error('version-number is invalid');
    }
}
export function createCliRunner(dependencies) {
    const { ensureCleanLocalGitState, getMergedPullRequests, createChangelog, packageInfo, prependFile } = dependencies;
    async function generateChangelog(options, githubRepo, validLabels, newVersionNumber) {
        if (!options.sloppy) {
            await ensureCleanLocalGitState(githubRepo);
        }
        const pullRequests = await getMergedPullRequests(githubRepo, validLabels);
        const changelog = createChangelog(newVersionNumber, validLabels, pullRequests, githubRepo);
        return stripTrailingEmptyLine(changelog);
    }
    return {
        run: async (newVersionNumber, options) => {
            const { repository } = packageInfo;
            if (!isRecord(repository)) {
                throw new Error('Repository information missing in package.json');
            }
            if (typeof repository.url !== 'string') {
                throw new TypeError('Repository url is not a string in package.json');
            }
            const githubRepo = getGithubRepo(repository.url);
            const validLabels = getValidLabels(packageInfo);
            validateVersionNumber(newVersionNumber);
            const changelog = await generateChangelog(options, githubRepo, validLabels, newVersionNumber);
            await prependFile(options.changelogPath, `${changelog.trim()}\n\n`);
        }
    };
}
//# sourceMappingURL=cli.js.map