import { format as formatDate } from 'date-fns';
import enLocale from 'date-fns/locale/en-US/index.js';
function formatLinkToPullRequest(pullRequestId, repo) {
    return `[#${pullRequestId}](https://github.com/${repo}/pull/${pullRequestId})`;
}
function formatPullRequest(pullRequest, repo) {
    return `* ${pullRequest.title} (${formatLinkToPullRequest(pullRequest.id, repo)})\n`;
}
function formatListOfPullRequests(pullRequests, repo) {
    return pullRequests
        .map((pr) => {
        return formatPullRequest(pr, repo);
    })
        .join('');
}
function formatSection(displayLabel, pullRequests, repo) {
    return `### ${displayLabel}\n\n${formatListOfPullRequests(pullRequests, repo)}\n`;
}
function isRecord(value) {
    return typeof value === 'object' && value !== null;
}
function getConfigValueFromPackageInfo(packageInfo, fieldName, fallback) {
    const prLogConfig = packageInfo['pr-log'];
    if (isRecord(prLogConfig)) {
        const field = prLogConfig[fieldName];
        if (typeof field === 'string') {
            return field;
        }
    }
    return fallback;
}
function groupByLabel(pullRequests) {
    return pullRequests.reduce((groupedObject, pullRequest) => {
        const { label } = pullRequest;
        const group = groupedObject[label];
        if (Array.isArray(group)) {
            return {
                ...groupedObject,
                [label]: [...group, pullRequest]
            };
        }
        return {
            ...groupedObject,
            [label]: [pullRequest]
        };
    }, {});
}
const defaultDateFormat = 'MMMM d, yyyy';
export function createChangelogFactory(dependencies) {
    const { getCurrentDate, packageInfo } = dependencies;
    const dateFormat = getConfigValueFromPackageInfo(packageInfo, 'dateFormat', defaultDateFormat);
    return function createChangelog(newVersionNumber, validLabels, mergedPullRequests, repo) {
        const groupedPullRequests = groupByLabel(mergedPullRequests);
        const date = formatDate(getCurrentDate(), dateFormat, { locale: enLocale });
        const title = `## ${newVersionNumber} (${date})`;
        let changelog = `${title}\n\n`;
        for (const [label, displayLabel] of validLabels) {
            const pullRequests = groupedPullRequests[label];
            if (pullRequests !== undefined) {
                changelog += formatSection(displayLabel, pullRequests, repo);
            }
        }
        return changelog;
    };
}
//# sourceMappingURL=create-changelog.js.map