import semver from 'semver';
export function getMergedPullRequestsFactory(dependencies) {
    const { gitCommandRunner, getPullRequestLabel } = dependencies;
    async function getLatestVersionTag() {
        const tags = await gitCommandRunner.listTags();
        const versionTags = tags.filter((tag) => {
            return semver.valid(tag) !== null && semver.prerelease(tag) === null;
        });
        const orderedVersionTags = versionTags.sort(semver.compare);
        const latestTag = orderedVersionTags.at(-1);
        if (latestTag === undefined) {
            throw new TypeError('Failed to determine latest version number git tag');
        }
        return latestTag;
    }
    async function getPullRequests(fromTag) {
        const mergeCommits = await gitCommandRunner.getMergeCommitLogs(fromTag);
        return mergeCommits.map((log) => {
            const matches = /^Merge pull request #(?<id>\d+) from .*?$/u.exec(log.subject);
            if (matches?.groups?.id === undefined) {
                throw new TypeError('Failed to extract pull request id from merge commit log');
            }
            if (log.body === undefined) {
                throw new TypeError('Failed to extract pull request title from merge commit log');
            }
            return { id: Number.parseInt(matches.groups.id, 10), title: log.body };
        });
    }
    async function extendWithLabel(githubRepo, validLabels, pullRequests) {
        const promises = pullRequests.map(async (pullRequest) => {
            const label = await getPullRequestLabel(githubRepo, validLabels, pullRequest.id, dependencies);
            return {
                id: pullRequest.id,
                title: pullRequest.title,
                label
            };
        });
        return Promise.all(promises);
    }
    return async function getMergedPullRequests(githubRepo, validLabels) {
        const latestVersionTag = await getLatestVersionTag();
        const pullRequests = await getPullRequests(latestVersionTag);
        const pullRequestsWithLabels = await extendWithLabel(githubRepo, validLabels, pullRequests);
        return pullRequestsWithLabels;
    };
}
//# sourceMappingURL=get-merged-pull-requests.js.map