import { oneLine } from 'common-tags';
import { splitByString, splitByPattern } from './split.js';
function trim(value) {
    return value.trim();
}
function isNonEmptyString(value) {
    return value.length > 0;
}
function splitLines(value, lineSeparator = '\n') {
    return splitByString(value, lineSeparator).map(trim).filter(isNonEmptyString);
}
const lineSeparator = '##$$@@$$##';
const fieldSeparator = '__||__';
function createParsableGitLogFormat() {
    const subjectPlaceholder = '%s';
    const bodyPlaceholder = '%b';
    const fields = [subjectPlaceholder, bodyPlaceholder];
    return `${fields.join(fieldSeparator)}${lineSeparator}`;
}
export function createGitCommandRunner(dependencies) {
    const { execute } = dependencies;
    return {
        async getShortStatus() {
            const result = await execute('git status --short');
            return result.stdout.trim();
        },
        async getCurrentBranchName() {
            const result = await execute('git rev-parse --abbrev-ref HEAD');
            return result.stdout.trim();
        },
        async fetchRemote(remoteAlias) {
            await execute(`git fetch ${remoteAlias}`);
        },
        async getSymmetricDifferencesBetweenBranches(branchA, branchB) {
            const result = await execute(`git rev-list --left-right ${branchA}...${branchB}`);
            return splitLines(result.stdout);
        },
        async getRemoteAliases() {
            const result = await execute('git remote -v');
            return splitLines(result.stdout).map((line) => {
                const remoteLineTokens = splitByPattern(line, /\s/);
                const [alias, url] = remoteLineTokens;
                if (url === undefined) {
                    throw new TypeError('Failed to determine git remote alias');
                }
                return { alias, url };
            });
        },
        async listTags() {
            const result = await execute('git tag --list');
            return splitLines(result.stdout);
        },
        async getMergeCommitLogs(from) {
            const result = await execute(oneLine `git log --first-parent --no-color
                    --pretty=format:${createParsableGitLogFormat()} --merges ${from}..HEAD`);
            const logs = splitLines(result.stdout, lineSeparator);
            return logs.map((log) => {
                const parts = splitByString(log, fieldSeparator);
                const [subject, body] = parts;
                return { subject, body: body === '' ? undefined : body };
            });
        }
    };
}
//# sourceMappingURL=git-command-runner.js.map